#include "PersonalAssistant.h"

PersonalAssistant::PersonalAssistant(DFRobotDFPlayerMini& player) {
  this->dfPlayer = &player;
}

void PersonalAssistant::numberToWord(int32_t number, uint8_t words[]) {
  // words[0] is reserved for array length
  uint8_t i = 1;
  if (number == 0) {
    this->number2word(0, &words[1]);
    words[0] = 2;
    return;
  }
  if (number < 0) {
    words[i++] = this->N2W_MINUS;
    number = abs(number);
  }

  uint8_t d12 = number % 100;
  uint8_t d3  = (number / 100) % 10;
  uint8_t d45 = (number / 1000) % 100;
  uint8_t d6  = (number / 100000) % 10;

  uint8_t buffer[2] = { this->N2W_NIL, this->N2W_NIL };

  if (d6 != 0) {
    this->number2word(d6, buffer);
    words[i++] = buffer[0];
    words[i++] = this->N2W_HUNDRED;
    if (d45 != 0) {
      words[i++] = this->N2W_AND;
    }
  }
  if (d45 != 0) {
    this->number2word(d45, buffer);
    if (buffer[0] != this->N2W_NIL)
      words[i++] = buffer[0];
    if (buffer[1] != this->N2W_NIL)
      words[i++] = buffer[1];
    words[i++] = this->N2W_THOUSAND;
  }
  if (d3 != 0) {
    this->number2word(d3, buffer);
    words[i++] = buffer[0];
    words[i++] = this->N2W_HUNDRED;
    if (d12 != 0) {
      words[i++] = this->N2W_AND;
    }
  }
  if (d12 != 0) {
    this->number2word(d12, buffer);
    if (buffer[0] != this->N2W_NIL)
      words[i++] = buffer[0];
    if (buffer[1] != this->N2W_NIL)
      words[i++] = buffer[1];
  }
  words[0] = i;
}

void PersonalAssistant::number2word(uint8_t number, uint8_t wheelbarrows[]) {
  if (number > 19) {
    wheelbarrows[0] = this->_tensMultiple[number / 10];
    wheelbarrows[1] = number % 10 ? this->_ones[number % 10] : this->N2W_NIL;
  } else if (number < 10) {
    wheelbarrows[0] = this->_ones[number];
  } else {
    wheelbarrows[0] = this->_tensExceptional[(number % 10) + 1];
  }
}

void PersonalAssistant::loadServices(File _services) {
  // for (uint8_t i=0; i<12; i++) {
  //   this->_moduleQueue[i] = true;
  // }return;
  //
  if (!_services)
    return;
  const size_t capacity = _services.size();
    if (capacity > 1024)
      return;

  std::unique_ptr<char[]> buf(new char[capacity]);
  _services.readBytes(buf.get(), capacity);
  DynamicJsonBuffer jsonBuffer(capacity);
  JsonObject& root = jsonBuffer.parseObject(buf.get());

  if (!root.success())
    return;

  JsonObject& weather = root["weather"];
  this->_moduleQueue[0] = root["gmail"]["unreadMessages"][0];
  this->_moduleQueue[1] = root["birthDeath"]["worldBirth"][0];
  this->_moduleQueue[2] = root["birthDeath"]["worldDeath"][0];
  this->_moduleQueue[3] = weather["currentTemperature"][0];
  this->_moduleQueue[4] = weather["todayCondition"][0];
  this->_moduleQueue[5] = weather["todayMinMax"][0];
  this->_moduleQueue[6] = weather["tomorrowCondition"][0];
  this->_moduleQueue[7] = weather["tomorrowMinMax"][0];
  this->_moduleQueue[8] = weather["precipitationForecast"][0];
  this->_moduleQueue[9] = weather["sunriseSunset"][0];
  this->_moduleQueue[10] = root["time"]["currentTime"][0];
  this->_moduleQueue[11] = root["time"]["currentDate"][0];

  _services.close();
}

void PersonalAssistant::saveServices(File _services) {
  if (!_services)
    return;

  uint8_t index = 0;

  const size_t capacity = 12*JSON_ARRAY_SIZE(2) + JSON_OBJECT_SIZE(1) + 2*JSON_OBJECT_SIZE(2) + JSON_OBJECT_SIZE(4) + JSON_OBJECT_SIZE(7);
  DynamicJsonBuffer jsonBuffer(capacity);

  JsonObject& root = jsonBuffer.createObject();

  JsonObject& gmail = root.createNestedObject(F("gmail"));

  JsonArray& gmail_unreadMessages = gmail.createNestedArray(F("unreadMessages"));
  gmail_unreadMessages.add(this->_moduleQueue[GMAIL_UNREADMESSAGES]);
  gmail_unreadMessages.add(++index);

  JsonObject& birthDeath = root.createNestedObject(F("birthDeath"));

  JsonArray& birthDeath_worldBirth = birthDeath.createNestedArray(F("worldBirth"));
  birthDeath_worldBirth.add(this->_moduleQueue[BIRTHDEATH_WORLDBIRTH]);
  birthDeath_worldBirth.add(++index);

  JsonArray& birthDeath_worldDeath = birthDeath.createNestedArray(F("worldDeath"));
  birthDeath_worldDeath.add(this->_moduleQueue[BIRTHDEATH_WORLDDEATH]);
  birthDeath_worldDeath.add(++index);

  JsonObject& weather = root.createNestedObject(F("weather"));

  JsonArray& weather_currentTemperature = weather.createNestedArray(F("currentTemperature"));
  weather_currentTemperature.add(WEATHER_CURRENTTEMPERATURE);
  weather_currentTemperature.add(++index);

  JsonArray& weather_todayCondition = weather.createNestedArray(F("todayCondition"));
  weather_todayCondition.add(this->_moduleQueue[WEATHER_TODAYCONDITION]);
  weather_todayCondition.add(++index);

  JsonArray& weather_todayMinMax = weather.createNestedArray(F("todayMinMax"));
  weather_todayMinMax.add(this->_moduleQueue[WEATHER_TODAYMINMAX]);
  weather_todayMinMax.add(++index);

  JsonArray& weather_tomorrowCondition = weather.createNestedArray(F("tomorrowCondition"));
  weather_tomorrowCondition.add(this->_moduleQueue[WEATHER_TOMORROWCONDITION]);
  weather_tomorrowCondition.add(++index);

  JsonArray& weather_tomorrowMinMax = weather.createNestedArray(F("tomorrowMinMax"));
  weather_tomorrowMinMax.add(this->_moduleQueue[WEATHER_TOMORROWMINMAX]);
  weather_tomorrowMinMax.add(++index);

  JsonArray& weather_precipitationForecast = weather.createNestedArray(F("precipitationForecast"));
  weather_precipitationForecast.add(this->_moduleQueue[WEATHER_PRECIPITATIONFORECAST]);
  weather_precipitationForecast.add(++index);

  JsonArray& weather_sunriseSunset = weather.createNestedArray(F("sunriseSunset"));
  weather_sunriseSunset.add(this->_moduleQueue[WEATHER_SUNRISESUNSET]);
  weather_sunriseSunset.add(++index);

  JsonObject& time = root.createNestedObject(F("time"));

  JsonArray& time_currentTime = time.createNestedArray(F("currentTime"));
  time_currentTime.add(this->_moduleQueue[TIME_CURRENTTIME]);
  time_currentTime.add(++index);

  JsonArray& time_currentDate = time.createNestedArray(F("currentDate"));
  time_currentDate.add(this->_moduleQueue[TIME_CURRENTDATE]);
  time_currentDate.add(++index);

  root.printTo(_services);

  _services.close();
}

void PersonalAssistant::setModule(uint8_t module, bool enable) {
  if (module < 12) {
    this->_moduleQueue[module] = enable;
  }
}

void PersonalAssistant::getDate(uint32_t time, uint16_t date[]) {
  const uint8_t monthDays[]={31,28,31,30,31,30,31,31,30,31,30,31}; // API starts months from 1, this array starts from 0
	uint16_t year;
	uint8_t month, day, hour, monthLength;
	uint32_t days;

	time /= 60; // in minutes
	time /= 60; // in hours
	hour = time % 24;
	time /= 24; // in days

	year = 0;
	days = 0;

	while((unsigned)(days += (LEAP_YEAR(year) ? 366 : 365)) <= time) {
		year++;
	}

	days -= LEAP_YEAR(year) ? 366 : 365;
	time  -= days; // now it is days in this year, starting at 0

	days=0;
	month=0;
	monthLength=0;
	for (month=0; month<12; month++) {
  	if (month==1) { // february
  		if (LEAP_YEAR(year)) {
  			monthLength=29;
  		} else {
  			monthLength=28;
  		}
  	} else {
  		monthLength = monthDays[month];
  	}

  	if (time >= monthLength) {
  		time -= monthLength;
  	} else {
  		break;
  	}
	}
	month = month + 1;  // jan is month 1
	day = time + 1;     // day of month
	year += 1970;

  date[0] = day;
  date[1] = month;
  date[2] = year;
}

void PersonalAssistant::handleSpeak() {
  if (!this->speakQisEmpty()) {
    if (millis() - this->speakTimestamp > this->durationQueue[(!this->speakQStart ? 31 : this->speakQStart-1)] || this->speakTimestamp == 0) {
      this->dfPlayer->playFolder(2, this->speakQLoad());
      this->speakTimestamp = millis();
    }
  }
}

uint8_t PersonalAssistant::getQueueTurn() {
  uint8_t start = this->_turn;
  do {
    if (this->_turn > 11) {
      this->_turn = 0;
    }
    if (this->_moduleQueue[this->_turn]) {
      return this->_turn++;
    }
    this->_turn++;
  } while(this->_turn != start);
  return 0xff;
}

void PersonalAssistant::speakQAdd(uint8_t item, uint16_t duration) {
  this->speakQueue[this->speakQEnd] = item;
  this->durationQueue[this->speakQEnd] = duration;
  if (++this->speakQEnd > 31)
    this->speakQEnd = 0;
}

uint8_t PersonalAssistant::speakQLoad() {
  if (++this->speakQStart > 31) {
    this->speakQStart = 0;
    return this->speakQueue[31];
  }
  return this->speakQueue[this->speakQStart-1];
}

void PersonalAssistant::speakQClear() {
  this->speakQStart = 0;
  this->speakQEnd = 0;
}

bool PersonalAssistant::speakQisEmpty() {
  return this->speakQStart == this->speakQEnd ? true : false;
}

uint8_t PersonalAssistant::calcNumSpeakDuration(uint8_t number) {
  if (number <= 0X04 || number == 0X08 || number == 0X10 || (number >= 0XA2 && number <= 0XA5) || (number >= 0XA8 && number <= 0XB3) || number == 0XC3) {
    return 58;
  } else if ((number <= 0X12 && number != 0X06) || number == 0X15 || number >= 0XA6) {
    return 68;
  } else if (number <= 0X14 || number == 0X19) {
    return 81;
  } else {
    return 96;
  }
}
