#include "Weather.h"


Weather::Weather() {

}

void Weather::parse(const char* response) {
    const size_t capacity = JSON_OBJECT_SIZE(10) + 180;
    DynamicJsonBuffer jsonBuffer(capacity);

    JsonObject& root = jsonBuffer.parseObject(response);

    this->_sunrise = root["sunrise"].as<String>();
    this->_sunset = root["sunset"].as<String>();

    this->_conditionCode = root["condition_code"];
    this->_conditionTemp = root["condition_temp"];

    this->_todayCode = root["today_code"];
    this->_todayHigh = root["today_high"];
    this->_todayLow = root["today_low"];

    this->_tomorrowCode = root["tomorrow_code"];
    this->_tomorrowHigh = root["tomorrow_high"];
    this->_tomorrowLow = root["tomorrow_low"];
}

bool Weather::isPrecipitation(uint16_t c) {
    if ( (c > 2 && c < 19) || c == 35 || (c > 36 && c < 44) || (c > 44 && c < 48) )
        return true;
    return false;
}

bool Weather::isPrecipitation() {
    return isPrecipitation(this->_tomorrowCode);
}

const char* Weather::getSunrise() {
    return this->_sunrise.c_str();
}

const char* Weather::getSunset() {
    return this->_sunset.c_str();
}

uint16_t Weather::getConditionCode() {
    return this->_conditionCode;
}

uint16_t Weather::getTodayCode() {
    return this->_todayCode;
}

uint16_t Weather::getTomorrowCode() {
    return this->_tomorrowCode;
}

int8_t Weather::getConditionTemp() {
    return this->_conditionTemp;
}

int8_t Weather::getTodayHigh() {
    return this->_todayHigh;
}

int8_t Weather::getTodayLow() {
    return this->_todayLow;
}

int8_t Weather::getTomorrowHigh() {
    return this->_tomorrowHigh;
}

int8_t Weather::getTomorrowLow() {
    return this->_tomorrowLow;
}

uint8_t Weather::getHours(const char* time) {
  if (time[1] == ':') {
    return time[0] - '0';
  } else if (time[2] == ':') {
    return ((time[0] - '0') * 10) + (time[1] - '0');
  }
  return 0xff;
}

uint8_t Weather::getMinutes(const char* time) {
  if (time[1] == ':') {
    return ((time[2] - '0') * 10) + (time[3] - '0');
  } else if (time[2] == ':') {
    return ((time[3] - '0') * 10) + (time[4] - '0');
  }
  return 0xff;
}
uint8_t Weather::getWeatherCondition(int8_t todayLow, int8_t todayHigh, int8_t tomorrowLow, int8_t tomorrowHigh) {
  uint8_t mode = MODE_NEUTRAL;

  /**
   ** PHASE 1
  **/
  if (todayLow < todayHigh && todayHigh < tomorrowLow && tomorrowLow < tomorrowHigh) {
    mode = MODE_WARMER;
      if (tomorrowHigh - todayHigh < 3) {
          mode = MODE_NEUTRAL;
      }
  }

  /**
   ** PHASE 1.5
  **/
  if (todayLow < todayHigh && todayHigh == tomorrowLow && tomorrowLow < tomorrowHigh) {
    mode = MODE_WARMER;
      if (tomorrowHigh - todayLow < 4) {
          mode = MODE_NEUTRAL;
      }
  }

  /**
   ** PHASE 2
  **/
  if (tomorrowLow < tomorrowHigh && tomorrowHigh < todayLow && todayLow < todayHigh) {
    mode = MODE_COLDER;
      if (todayLow - tomorrowLow < 3) {
          mode = MODE_NEUTRAL;
      }
  }

  /**
   ** PHASE 2.5
  **/
  if (tomorrowLow < tomorrowHigh && todayLow == tomorrowHigh && todayLow < todayHigh) {
    mode = MODE_COLDER;
      if (todayHigh - tomorrowLow < 4) {
          mode = MODE_NEUTRAL;
      }
  }

  /**
   ** PHASE 3
  **/
  if (todayLow < tomorrowLow && tomorrowLow < tomorrowHigh && tomorrowHigh < todayHigh) {
      if ((tomorrowLow - todayLow < 3) && (todayHigh - tomorrowHigh < 3)) {
          mode = MODE_NEUTRAL;
      } else if ((tomorrowLow - todayLow) < (todayHigh - tomorrowHigh)) {
          mode = MODE_COLDER;
      } else {
          mode = MODE_WARMER;
      }
  }

  /**
   ** PHASE 4
  **/
  if (tomorrowLow < todayLow && todayLow < todayHigh && todayHigh < tomorrowHigh) {
      if ((todayLow - tomorrowLow < 3) && (tomorrowHigh - todayHigh < 3)) {
          mode = MODE_NEUTRAL;
      } else if ((todayLow - tomorrowLow) <= (tomorrowHigh - todayHigh)) {
          mode = MODE_WARMER;
      } else {
          mode = MODE_COLDER;
      }
  }

  /**
   ** PHASE 5
  **/
  if (todayLow < tomorrowLow && tomorrowLow < todayHigh && todayHigh < tomorrowHigh) {
    mode = MODE_WARMER;
      if ((tomorrowHigh - todayHigh < 3) && (tomorrowLow - todayLow < 3)) {
          mode = MODE_NEUTRAL;
      }
  }

  /**
   ** PHASE 6
  **/
  if (tomorrowLow < todayLow && todayLow < tomorrowHigh && tomorrowHigh < todayHigh) {
    mode = MODE_COLDER;
      if ((todayLow - tomorrowLow < 3) && (todayHigh - tomorrowHigh < 3)) {
          mode = MODE_NEUTRAL;
      }
  }

  /**
   ** PHASE 7
  **/
  if (todayLow == tomorrowLow && todayHigh == tomorrowHigh) {
      mode = MODE_NEUTRAL;
  }

  /**
   ** PHASE 8
  **/
  if (todayLow == tomorrowLow && todayHigh > tomorrowHigh) {
    mode = MODE_COLDER;
      if (todayHigh - tomorrowHigh < 3) {
          mode = MODE_NEUTRAL;
      }
  }

  /**
   ** PHASE 9
  **/
  if (todayLow == tomorrowLow && tomorrowHigh > todayHigh) {
    mode = MODE_WARMER;
      if (tomorrowHigh - todayHigh < 3) {
          mode = MODE_NEUTRAL;
      }
  }

  /**
   ** PHASE 10
  **/
  if (todayHigh == tomorrowHigh && tomorrowLow > todayLow) {
    mode = MODE_WARMER;
      if (tomorrowLow - todayLow < 3) {
          mode = MODE_NEUTRAL;
      }
  }

  /**
   ** PHASE 11
  **/
  if (todayHigh == tomorrowHigh && todayLow > tomorrowLow) {
    mode = MODE_COLDER;
      if (todayLow - tomorrowLow < 3) {
          mode = MODE_NEUTRAL;
      }
  }

  return mode;
}
